/*
 * @OSF_COPYRIGHT@
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993, 1994, 1996 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (DCE).
 * Copyright (c) Hewlett-Packard Company 1994
*/

#ifndef __pwd_cache_h_included__
#define __pwd_cache_h_included__

#include <dce/passwd.h>
#include <dce/rsec_pwd_mgmt.h>
#include <pthread.h>
#include <time.h>

/*
 * cache_size is the size of the table
 * 100 is the default.
 */
extern signed32 cache_size;

/*
 * timeout is the maximum number of seconds a generated
 * password should stay in the cache. Any strength checks
 * on a generated password longer than this timeout should
 * automatically fail!
 */
extern signed32 timeout;

/*
 * cache_mutex is a pthread mutex lock used when a thread
 * is accessing the cache. It is used for writing *and*
 * reading because writing while someone is reading can
 * be disasterous.
 */
extern pthread_mutex_t cache_mutex;

/*
 * struct which holds the generated passwords
 */
typedef struct field {
    int              num_pwds;
    sec_passwd_rec_t **pwd;
} *field_t;

typedef field_t *field_p_t;

/*
 * struct which holds a record containing generated
 * passwords for a user.
 */
typedef struct record {
    char                    *who;
    time_t                  last_update;
    unsigned32              num_pwds;
    sec_passwd_rec_t        *gen_pwd_set;
    struct record           *next;    /* for duplicate hashvals */
} rec_t;

typedef rec_t *rec_p_t;

/*
 * table holds all of the password records
 */
rec_p_t *table;

/* function prototypes */
void create_table(error_status_t *);
static unsigned hash(const char *);
void create_record (
    char *, 
    unsigned32,
    sec_passwd_rec_t *, 
    error_status_t *
);
void cleanup_table(error_status_t *);
boolean32 check_cache(
    char *, 
    sec_passwd_rec_t *, 
    boolean32 *, 
    error_status_t *
);

#endif
